/**
 * Handle task command.
 * @function apemanTsk
 * @memberof module:apeman-task/lib
 */

"use strict";

var argx = require('argx'),
    arrayreduce = require('arrayreduce'),
    arrayfilter = require('arrayfilter'),
    runTask = require('./tasking/run_task'),
    listTasks = require('./tasking/list_tasks'),
    apemanfile = require('apemanfile'),
    async = require('async');

/** @lends apemanTsk */
function apemanTsk(name, options, callback) {
    var args = argx(arguments);
    callback = args.pop('function') || argx.noop;
    options = args.pop('object') || {};
    name = args.remain().reduce(arrayreduce.arrayConcat(), []).filter(arrayfilter.emptyReject());
    async.waterfall([
        function (callback) {
            apemanfile.load(options.configuration, callback);
        },
        function (apemanfile, callback) {
            var here = process.cwd(),
                cwd = apemanfile.$cwd || process.cwd();
            async.series([
                function (callback) {
                    process.chdir(cwd);
                    callback(null);
                },
                function (callback) {
                    if (name.length > 0) {
                        runTask(apemanfile, name, {
                            verbose: options.verbose
                        }, callback);
                    } else {
                        listTasks(apemanfile, callback);
                    }
                },
                function (callback) {
                    process.chdir(here);
                    callback(null);
                }
            ], callback);

        }
    ], callback);
}

module.exports = apemanTsk;
