/**
 * List available tasks.
 * @memberof module:apeman-task
 * @function listTasks
 * @param {Apemanfile} - Apemanfile to work with.
 * @param {object} options - Optional settings.
 * @param {function} callback - Callback when done.
 */
"use strict";

var argx = require('argx'),
    extend = require('extend'),
    async = require('async'),
    ListLogger = require('../logging/list_logger');

/** @lends listTasks */
function listTasks(apemanfile, options, callback) {
    var args = argx(arguments);
    apemanfile = args.shift();
    callback = args.pop('function') || argx.noop;
    options = args.pop('object') || {};
    async.waterfall([
        function (callback) {
            apemanfile.find("$tasks", callback);
        },
        function (tasks, callback) {
            var $children = apemanfile.$children || {};
            async.concatSeries(Object.keys($children), function (name, callback) {
                var $child = $children[name];
                async.waterfall([
                    function (callback) {
                        $child.find("$tasks", callback);
                    },
                    function (found, callback) {
                        var result = {
                            namespace: name,
                            tasks: found,
                            cwd: $child.$cwd
                        };
                        callback(null, result);
                    }
                ], callback);
            }, function (err, childTasks) {
                if (!err) {
                    var logger = new ListLogger({
                        verbose: options.verbose
                    });
                    logger.logList(tasks, childTasks || {});
                }
                callback(err);
            });
        }
    ], callback);
}

module.exports = listTasks;
